<?php

class predictionActions extends sfActions {

    public function executeIndex(sfWebRequest $request) {
        $this->userSettings = UserSettingsPeer::retrieveByUserId($this->getUser()->getUserId());
        InnovationUtil::forward404Unless($this->userSettings && $this->userSettings->getCompany(), $this, 'W ustawieniach nie została wybrana firma');
        $this->companyId = $this->userSettings->getCurrentcompanyid();

        $questionNumber = $request->getParameter(InnovationConstants::QUESTION_NUMBER);
        InnovationUtil::forward404Unless($questionNumber, $this, 'Nie podano numeru pytania, które ma być prognozowane');
        $questionNumber = trim($questionNumber);
        $questionNumber = explode('.', $questionNumber);
        InnovationUtil::forward404Unless(count($questionNumber) == 2 && InnovationValidator::isNumeric($questionNumber[0]) && InnovationValidator::isNumeric($questionNumber[1]), $this, 'Nie podano numeru pytania, które ma być prognozowane');
        
        $this->Question = QuestionPeer::retrieveByNumberAndSubnumber($questionNumber[0], $questionNumber[1]);
        InnovationUtil::forward404Unless($this->Question, $this, 'Pytanie o podanym numerze nie istnieje');
        InnovationUtil::forward404Unless($this->Question->getPredictionquestion() == 1, $this, 'Pytanie o podanym numerze nie może być prognozowane');
        //$this->Questions = QuestionPeer::retrieveByTypes(array(1, 2, 3, 4, 7, 8));
        $this->Questions = QuestionPeer::getQuestionsToPredict();
        $this->Predictions = PredictionPeer::retrieveByCompanyId($this->companyId);
        
        $this->startYear = $this->userSettings->getStartyear();
        $this->endYear = $this->userSettings->getEndyear();
        if (abs($this->startYear - $this->endYear) > 10) {
            $this->startYear = $this->endYear - 10;
        }
        
        $prediction = new Prediction();
        $prediction->setCompanyid($this->companyId);
        $prediction->setStartyear($this->startYear);
        $prediction->setEndyear($this->endYear);
        $prediction->setFieldname($this->Question->getQuestionFullNumber());
        
        if($this->Question->getType() == 3 || $this->Question->getType() == 4) {
             $this->endYear = $this->endYear - 1;
        }
        $this->innovationRange = $this->endYear - $this->startYear;
        
        $answers = InnovationUtil::getNumericQuestionAnswers($this->Question, $this->companyId, $this->startYear, $this->endYear);
        $factors = InnovationUtil::getLinearTrendFactors($answers, $this->startYear, $this->endYear);
        $this->predictions = InnovationUtil::getLinearTrendPredictions($factors, $answers, $this->startYear, $this->endYear);
        $this->predictionYears = InnovationUtil::getPredictionYears($this->Question->getType(), $this->startYear, $this->endYear);
        $this->getUser()->setFlash(InnovationConstants::PREDICTION_YEARS_PARAM_NAME, $this->predictionYears);
        
        $prediction->setYear1(number_format($this->predictions[0][$this->innovationRange + 1], 2, '.', ' '));
        $prediction->setYear2(number_format($this->predictions[0][$this->innovationRange + 2], 2, '.', ' '));
        $prediction->setYear3(number_format($this->predictions[0][$this->innovationRange + 3], 2, '.', ' '));
        $prediction->setYear4(number_format($this->predictions[0][$this->innovationRange + 4], 2, '.', ' '));
        $prediction->setYear5(number_format($this->predictions[0][$this->innovationRange + 5], 2, '.', ' '));
        $prediction->setTypeoftrend(InnovationConstants::LINEAR_TREND_TYPE);
        
        $this->answers = $answers;
        $this->form = new PredictionForm($prediction);
    }
    
    public function executeDelete(sfWebRequest $request) {
        $request->checkCSRFProtection();

        $this->forward404Unless($Prediction = PredictionPeer::retrieveByPk($request->getParameter('predictionid')), sprintf('Object Prediction does not exist (%s).', $request->getParameter('predictionid')));
        $Prediction->delete();

        $questionNumber = $request->getParameter(InnovationConstants::QUESTION_NUMBER);
        $this->redirect('prediction/index?'.InnovationConstants::QUESTION_NUMBER.'='.$questionNumber);
    }
    
    public function executeCreate(sfWebRequest $request) {
        $this->forward404Unless($request->isMethod(sfRequest::POST));

        $this->form = new PredictionForm();
        $this->processForm($request, $this->form);

        $questionNumber = $request->getParameter(InnovationConstants::QUESTION_NUMBER);
        $this->redirect('prediction/index?'.InnovationConstants::QUESTION_NUMBER.'='.$questionNumber);
    }
    
    public function executeReport() {
    	/// Brzydki kawał kodu dopisywany na siłę
    	ob_clean();
    	
    	$userSettings = UserSettingsPeer::retrieveByUserId($this->getUser()->getUserId());
    	 
    	$company = $userSettings->getCompany();
    	 
    	InnovationUtil::forward404Unless($userSettings && $company, $this, 'W ustawieniach nie została wybrana firma');
    	    	
    	$config = sfTCPDFPluginConfigHandler::loadConfig();
    	$i18n = $this->getContext()->getI18N();
    	
    	
    	// pdf object
    	$pdf = new sfTCPDF();
    	 
    	// set document information
    	$pdf->SetCreator('SPiWIN Education');
    	$pdf->SetTitle($i18n->__('Ocena jakości innowacji'));
    	
    	$pdf->setHeaderMargin(10);
    	$pdf->setHeaderData('', 0, '', $i18n->__('Politechnika Warszawska'));
    	$pdf->setHeaderFont(array('dejavusans', '', 8));
    	$pdf->headerCallback = 'predictionActions::addReportHeader';
    	
    	$pdf->setFooterMargin(25);
        //$pdf->setFooterData('', 0, '', $i18n->__('Praca Inżynierska'));
    	$pdf->setFooterFont(array('dejavusans', '', 8));
    	$pdf->footerCallback = 'predictionActions::addReportFooter';
    	
    	$pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);
    	$pdf->SetMargins(PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT);
    	 
    	//set auto page breaks
    	$pdf->SetAutoPageBreak(TRUE, PDF_MARGIN_BOTTOM);
    	 
    	//set image scale factor
    	$pdf->setImageScale(PDF_IMAGE_SCALE_RATIO);
    	 
    	// ---------------------------------------------------------
    	 
    	// set default font subsetting mode
    	$pdf->setFontSubsetting(true);
    	
    	$pdf->AddPage();
    	
    	$pdf->SetFont('dejavusans', '', 30, '', true);
    	
    	// Set some content to print
    	$html = '<div style="text-align: center; margin-top: 200px">'.
    		$i18n->__('OCENA JAKOŚCI INNOWACJI').'</div>';
    	// Print text using writeHTMLCell()
    	$pdf->writeHTMLCell($w=0, $h=0, $x='', $y=60, $html, $border=0, $ln=1, $fill=0, $reseth=true, $align='', $autopadding=true);
    	
    	$pdf->SetFont('dejavusans', '', 25);
    	
    	$html = $i18n->__('Raport z badania');
    	$pdf->writeHTMLCell($w=0, $h=0, $x=35, $y=90, $html, $border=0, $ln=1, $fill=0, $reseth=true, $align='', $autopadding=true);
    	
    	$pdf->SetFont('dejavusans', '', 19);
    	
    	$html = $i18n->__('Oceny innowacyjności przedsiębiorstwa');
    	$pdf->writeHTMLCell($w=0, $h=0, $x=35, $y=100, $html, $border=0, $ln=1, $fill=0, $reseth=true, $align='', $autopadding=true);
    	
    	$pdf->writeHTMLCell($w=0, $h=0, $x=35, $y=110, $company->getName(), $border=0, $ln=1, $fill=0, $reseth=true, $align='', $autopadding=true);
    	
    	$pdf->AddPage();

    	$pdf->Cell(0, 10, $i18n->__('Ocena skuteczności innowacji'));
    	$pdf->Ln();
    	$pdf->Ln();
    	$pdf->SetFont('dejavusans', '', 12);
    	
    	// Nagłówek tabeli
    	$html = $this->tableHead.
    	'<table><thead><tr><th>'.
    	$i18n->__('L.p.').
    	'</th><th>'.
    	$i18n->__('Temat innowacyjny planowany do realizacji').
    	'<br/> ['.$i18n->__('wykonania').
    	'] <br/> \'W\'</th><th>'.
    	$i18n->__('Termin realizacji').'<br/> \'T\'</th><th>'.
    	$i18n->__('Nakłady').'<br/> \'N\'</th><th>'.
    	$i18n->__('Odpowiedzialność')
    	.'<br/> \'O\'</th></tr></thead>';
    	
    	$html .='<tbody>';

    	$arr = InnovationUtil::computeEfficacy($this->getUser()->getUserId(), $this);
    	
    	$companyEfficacy = $arr[0];
    	$efficacyCount = $arr[1];
    	$sumEfficacy = $arr[2];
    	
    	$rowNum = 0;
    	
    	for ($i = 0; $i < $efficacyCount; $i++) {
    		$rowNum = $i;
    		$html.= '<tr'.$this->trStyle($rowNum).'><td>'.($i + 1).'</td>'.
                    '<td>'.($companyEfficacy[$i]->getWw() < 0 ? '-' : $companyEfficacy[$i]->getWw()).'</td>'.
                    '<td>'.($companyEfficacy[$i]->getWt() < 0 ? '-' : $companyEfficacy[$i]->getWt()).'</td>'.
                    '<td>'.($companyEfficacy[$i]->getWn() < 0 ? '-' : $companyEfficacy[$i]->getWn()).'</td>'.
                    '<td>'.($companyEfficacy[$i]->getWo() < 0 ? '-' : $companyEfficacy[$i]->getWo()).'</td>'.
                '</tr>';
    	}
    	
    	$rowNum++;
    	
    	$html.='<tr'.$this->trStyle($rowNum).'>';
    	
    	if ($efficacyCount == 1) {
    		$html.= '<td>Σ</td>'.
                    '<td>'.($companyEfficacy[0]->getWw() < 0 ? '-' : number_format($companyEfficacy[0]->getWw(), 5, '.', ' ')).'</td>'.
                    '<td>'.($companyEfficacy[0]->getWt() < 0 ? '-' : number_format($companyEfficacy[0]->getWt(), 5, '.', ' ')).'</td>'.
                    '<td>'.($companyEfficacy[0]->getWn() < 0 ? '-' : number_format($companyEfficacy[0]->getWn(), 5, '.', ' ')).'</td>'.
                    '<td>'.($companyEfficacy[0]->getWo() < 0 ? '-' : number_format($companyEfficacy[0]->getWo(), 5, '.', ' ')).'</td>';
    	} else {
    		$html.='<td>((Σ(Wx1 ÷ Wx'.$efficacyCount.')) / '.$efficacyCount.')</td>'.
                    '<td>'.($sumEfficacy->getWw() < 0 ? '-' : number_format($sumEfficacy->getWw(), 5, '.', ' ')).'</td>'.
                    '<td>'.($sumEfficacy->getWt() < 0 ? '-' : number_format($sumEfficacy->getWt(), 5, '.', ' ')).'</td>'.
                    '<td>'.($sumEfficacy->getWn() < 0 ? '-' : number_format($sumEfficacy->getWn(), 5, '.', ' ')).'</td>'.
                    '<td>'.($sumEfficacy->getWo() < 0 ? '-' : number_format($sumEfficacy->getWo(), 5, '.', ' ')).'</td>';
    	}
    	
    	$rowNum++;
    	
    	$html.='</tr><tr'.$this->trStyle($rowNum).'>'.
    	       '<td colspan="2">WSI='.($sumEfficacy->getWProduct() < 0 ? ' - ' : number_format($sumEfficacy->getWProduct(), 5, '.', ' ')).'</td>'.
    	       '<td colspan="2">'.$i18n->__('Rodzaj skuteczności innowacji').'</td>'.
    	       '<td>';
    	if ($sumEfficacy->getEffectivenessType() < 1) {
    		$html.='-';
    	} elseif ($sumEfficacy->getEffectivenessType() == 1) {
    		$html.=$i18n->__('skuteczność bardzo wysoka');
    	} elseif ($sumEfficacy->getEffectivenessType() == 2) {
    		$html.=$i18n->__('skuteczność wysoka');
    	} elseif ($sumEfficacy->getEffectivenessType() == 3) {
    		$html.=$i18n->__('skuteczność dobra');
    	} elseif ($sumEfficacy->getEffectivenessType() == 4) {
    		$html.=$i18n->__('skuteczność średnia');
    	} elseif ($sumEfficacy->getEffectivenessType() == 5) {
    		$html.=$i18n->__('skuteczność niska');
    	} elseif ($sumEfficacy->getEffectivenessType() > 5) {
    		$html.=$i18n->__('skuteczność bardzo słaba');
    	}
    	$html.='</td></tr>';
    	
    	$html.='</tbody>';
    	
    	$html.='</table>';
    	$pdf->Ln();
    	$pdf->writeHTML($html, true, false, false, false, '');
    	
    	$pdf->SetFont('dejavusans', '', 19);
    	$pdf->Cell(0, 10, $i18n->__('Ocena efektywności bezpośredniej innowacji'));
    	$pdf->SetFont('dejavusans', '', 12);
    	$effectivenessResult = InnovationUtil::computeEffectiveness($this->getUser()->getUserId(), $this);
    	
    	$tab2 = $effectivenessResult['tab2'];
    	$startYear = $effectivenessResult['startYear'];
    	$endYear = $effectivenessResult['endYear'];
    	$innovationRange = $effectivenessResult['innovationRange'];
    	
    	$html = $this->tableHead.'<table><thead>';
    	$html .= '<tr><th>'.$i18n->__('L.p.').'</th>'.
    	         '<th>'.$i18n->__('Cykl innowacji').'<br/>['.$i18n->__('okres %n% letni', array('%n%' => $innovationRange)).']</th>'.
    	         '<th>'.$i18n->__('Dane z Tab.%nr%', array('%nr%' => '1')).'<br/>'.$i18n->__('poz. %nr%', array('%nr%' => '41')).',<br/>'.$i18n->__('suma z lat').':</th>'.
    	         '<th>'.$i18n->__('Dane z Tab.%nr%', array('%nr%' => '1')).'<br/>'.$i18n->__('poz. %nr%', array('%nr%' => '38.2')).',<br/>'.$i18n->__('suma z lat').':</th>'.
    	         '<th>'.$i18n->__('Dane z Tab.%nr%', array('%nr%' => '1')).'<br/>'.$i18n->__('poz. %nr%', array('%nr%' => '68.2')).',<br/>'.$i18n->__('suma z lat').':</th>'.
    	         '<th>'.$i18n->__('Efekty bezpośrednie').'<br/>['.$i18n->__('tys.').$i18n->__('zł').']</th></tr>';
    	$html .='</thead><tbody>';
    	
    	for ($i = $startYear; $i < $endYear + 2; $i++)
    	{
    		$html.='<tr'.$this->trStyle($i-$startYear).'>'.
    		'<td>'.($i - $startYear + 1). '.</td>'.
    		'<td>'.$i . ' - ' . ($i + $innovationRange - 1).'</td>'.
    		'<td>'.$tab2['41.0'][$i].'</td>'.
    		'<td>'.$tab2['38.2'][$i].'</td>'.
    		'<td>'.$tab2['68.2'][$i].'</td>'.
    		'<td>'.$tab2['sum'][$i].'</td>'.
    		'</tr>';
    	}
    	
    	$html .= '</tbody></table>';
    	$pdf->Ln();
    	$pdf->Ln();
    	$pdf->writeHTML($html, true, false, false, false, '');
    	
    	$html = $this->tableHead.'<table><thead>';
    	$html.='<tr><th>'.$i18n->__('L.p.').'</th>'.
    	       '<th>'.$i18n->__('Cykl innowacji').
    	       		'<br/>['.$i18n->__('okres %n% letni', array('%n%' => $innovationRange)).']</th>'.
    	       '<th>'.$i18n->__('Dane z Tab.%nr%', array('%nr%' => '1')).
    	       		'<br/>-'.$i18n->__('poz. %nr%', array('%nr%' => '1')).',<br/>'.$i18n->__('suma z lat').
    	       		'<br/>['.$i18n->__('Nakłady').']<br/>\'N\'</th>'.    	       		
    	       '<th>'.$i18n->__('Dane z Tab.%nr%', array('%nr%' => '2')).
    	       		'<br/>-'.$i18n->__('kolumna %nr%', array('%nr%' => '6')).
    	       		'<br/>\'E\'</th>'.
    	       '<th>[ E/N ] * 100% <br/>[ ( 4/3 ) * 100% ]</th>'.    	       
    	       '<th>%'.$i18n->__('udziału czasu realizacji cyklu innowacji').'–'.
    				$i18n->__('stan po zakończonym %r1% r. [ w roku %r2% ]', array('%r1%' => $endYear, '%r2%' => $endYear + 1)).'</th>'.
    	       '<th>'.$i18n->__('Stopień realizacji innowacji').
    				'[ % ]<br/>['.$i18n->__('poz. %nr%', array('%nr%' => '5')).'/'.
    				$i18n->__('poz. %nr%', array('%nr%' => '6')).'] * 100%</th>'.
    			'</tr></thead><tbody>';
    	for ($tmp = array(), $i = $startYear; $i < $endYear + 2; $i++){
    		$html.='<tr'.$this->trStyle($i-$startYear).'><td>'.($i - $startYear + 1). '.</td>'.
	    				'<td>'.$i .' - '.($i + $innovationRange - 1).'</td>'.
	    				'<td>'.$tab2['1.0'][$i].'</td>'.
    					'<td>'.$tab2['sum'][$i].'</td><td>';
    		               
    		if ($tab2['1.0'][$i] == 0 || $tab2['sum'][$i] == 0) {
    			$html.='0%';
    			$tmp['col5'][$i] = 0;
    		} else {
    			$tmp['col5'][$i] = $tab2['sum'][$i] / $tab2['1.0'][$i];
    			$html.=number_format($tmp['col5'][$i] * 100, 2, '.', ' ') . '%';
    		}
    		$html.='</td>';
    		$html.='<td>';
			$tmp['col6'][$i] = ($endYear - $i + 1) / $innovationRange;
    		$html.=(number_format($tmp['col6'][$i] * 100, 2, '.', ' ')) . '%</td>';
    		$html.='<td>';
    		
    		if ($tmp['col5'][$i] == 0 || $tmp['col6'][$i] == 0) {
    			$html.='0%';
    			$tmp['col7'][$i] = 0;
    		} else {
    			$tmp['col7'][$i] = $tmp['col5'][$i] / $tmp['col6'][$i];
    			$html.=(number_format($tmp['col7'][$i] * 100, 2, '.', ' ')) . '%';
    		}
    		
    		$html.='</td></tr>';
    	}
    	$html .= '</tbody></table>';
    	$pdf->Ln();
    	$pdf->Ln();
    	$pdf->writeHTML($html, true, false, false, false, '');

    	// Close and output PDF document
    	// This method has several options, check the source code documentation for more information.
    	$pdf->Output('report.pdf', 'I');
    	 
    	// Stop symfony process
    	throw new sfStopException();
    }
    
    private $tableHead =
    	'<style>
    		* {
    			font-size: 90px;
				color: #55737B;
    		}
    		table {
    			width: 100%;
    			margin-bottom: 18px;
    			border: 1px solid #D0D0D0;
    			border-collapse: separate;
    		}
    		td {
    			border-left: 1px solid #D0D0D0;
    			border-top: 1px solid #DDD;
    			padding: 8px;
    			text-alignn: left;
    			line-height: 18px;
    			vertical-align: top;
    		}
    		
    		th {
    			border-left: 1px solid #D0D0D0;
    			padding: 8px;
    			text-alignn: left;
    			line-height: 18px;
    			vertical-align: top;
    			font-weight: bold;
    		}
    	</style>'; 
    
    private function trStyle($i) {
    	$result = ' style="';
    	if (($i % 2) == 0) {
    		$result.='background-color:#F9F9F9;';
    	}
    	
    	$result.='" ';
    	
    	return $result;
    }
    
    public static function addReportHeader(sfTCPDF $pdf) {
    	$headerData = $pdf->getHeaderData();
    	$header = $headerData['string'];
    	$pdf->Cell(0, 10, $header, 0, false, 'C', 0, '', 0, false, 'T', 'M');  	
    }
    
    public static function addReportFooter(sfTCPDF $pdf) {
    	$footer = 'Praca Inżynierska';
        $pdf->Cell(0, 10, $footer, 0, false, 'C', 0, '', 0, false, 'T', 'M');    	
    }
    
    protected function processForm(sfWebRequest $request, sfForm $form) {
        $form->bind($request->getParameter($form->getName()), $request->getFiles($form->getName()));
        if ($form->isValid()) {
            $Prediction = $form->save();
        }
    }

}
