<?php

class AnswersCollectionForm extends sfForm {

    public function configure() {
        if (!$questions = $this->getOption('questions')) {
            throw new InvalidArgumentException('You must provide a questions object.');
        }

        $startYear = $this->getOption('startYear');
        $endYear = $this->getOption('endYear');
        $companyId = $this->getOption('companyId');

        if (!$startYear || !$endYear || !$companyId) {
            throw new InvalidArgumentException('You must provide a startYear, endYear and companyId.');
        }

        $nameFormat = 'collection_form';
        $this->getWidgetSchema()->setNameFormat($nameFormat.'[%s]');
        
        foreach ($questions as $question) {
            $nameQuestionFormat = $nameFormat.'['.$question->getQuestionid().'][';
            if ($question->getType() == 1 || $question->getType() == 7 || $question->getType() == 8) {
                
                $answerYearsSubForm = new sfForm();
                for ($i = $startYear; $i <= $endYear; $i++) {

                    $answerYears = AnswerYearsPeer::retrieveByQuestionIdCompanyIdYear($question->getQuestionid(), $companyId, $i);
                    $answerYearsForm = new AnswerYearsForm($answerYears, array('type' => $question->getType(), 'requiredQuestion' => $question->getRequiredquestion()));
                    $answerYearsForm->getWidgetSchema()->setNameFormat($nameQuestionFormat.$i.'][%s]');
                    $answerYearsSubForm->embedForm($i, $answerYearsForm);
                }
                $this->embedForm($question->getQuestionid(), $answerYearsSubForm);
                
            } elseif ($question->getType() == 3) {

                $answerYearsRangeSubForm = new sfForm();
                for ($i = $startYear; $i <= $endYear - 1; $i++) {

                    $answerYearsRange = AnswerYearsRangePeer::retrieveByQuestionIdCompanyIdYear($question->getQuestionid(), $companyId, $i);
                    $answerYearsRangeForm = new AnswerYearsRangeForm($answerYearsRange, array('requiredQuestion' => $question->getRequiredquestion()));
                    $answerYearsRangeForm->getWidgetSchema()->setNameFormat($nameQuestionFormat.$i.'][%s]');
                    $answerYearsRangeSubForm->embedForm($i, $answerYearsRangeForm);
                }
                $this->embedForm($question->getQuestionid(), $answerYearsRangeSubForm);
                
            } elseif ($question->getType() == 5) {

                $answerListSubForm = new sfForm();
                for ($i = $startYear; $i <= $endYear; $i++) {

                    $answerList = AnswerListPeer::retrieveByQuestionIdCompanyIdYear($question->getQuestionid(), $companyId, $i);
                    $answerListForm = new AnswerListForm($answerList, array('requiredQuestion' => $question->getRequiredquestion()));
                    $answerListForm->getWidgetSchema()->setNameFormat($nameQuestionFormat.$i.'][%s]');
                    $answerListSubForm->embedForm($i, $answerListForm);
                }
                $this->embedForm($question->getQuestionid(), $answerListSubForm);
                
            } elseif ($question->getType() == 6) {

                $answerMixedSubForm = new sfForm();

                foreach (AnswerMixedPeer::retrieveByNumberAndSubnumber($question->getNumber(), $question->getSubnumber()) as $answerMixed) {

                    if ($answerMixed->getType() == 1) {

                        $answerCheckbox = AnswerCheckboxPeer::retrieveByMixIdCompanyId($answerMixed->getAnswermixedid(), $companyId);
                        $answerCheckboxForm = new AnswerCheckboxForm($answerCheckbox);
                        $answerCheckboxForm->getWidgetSchema()->setNameFormat($nameQuestionFormat.$answerMixed->getAnswermixedid().'][%s]');
                        $answerMixedSubForm->embedForm($answerMixed->getAnswermixedid(), $answerCheckboxForm);
                        
                    } elseif ($answerMixed->getType() == 2) {
                        
                        $answerText =  AnswerTextPeer::retrieveByMixIdCompanyId($answerMixed->getAnswermixedid(), $companyId);
                        $answerTextForm = new AnswerTextForm($answerText, array('requiredQuestion' => $question->getRequiredquestion()));
                        $answerTextForm->getWidgetSchema()->setNameFormat($nameQuestionFormat.$answerMixed->getAnswermixedid().'][%s]');
                        $answerMixedSubForm->embedForm($answerMixed->getAnswermixedid(), $answerTextForm);
                    }
                }

                $this->embedForm($question->getQuestionid(), $answerMixedSubForm);
            }
        }
    }

}
?>
